@@ -1,6 +1,6 @@
 /*
-** FAAD - Freeware Advanced Audio Decoder
-** Copyright (C) 2002 M. Bakker
+** FAAD2 - Freeware Advanced Audio (AAC) Decoder including SBR decoding
+** Copyright (C) 2003-2005 M. Bakker, Nero AG, http://www.nero.com
 **  
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -16,7 +16,18 @@
 ** along with this program; if not, write to the Free Software 
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **
-** $Id: drc.h,v 1.2 2002/02/18 10:01:05 menno Exp $
+** Any non-GPL usage of this software or parts of this software is strictly
+** forbidden.
+**
+** Software using this code must display the following message visibly in or
+** on each copy of the software:
+** "FAAD2 AAC/HE-AAC/HE-AACv2/DRM decoder (c) Nero AG, www.nero.com"
+** in, for example, the about-box or help/startup screen.
+**
+** Commercial non-GPL licensing of this software is possible.
+** For more info contact Nero AG through Mpeg4AAClicense@nero.com.
+**
+** $Id: drc.h,v 1.20 2006/05/07 18:09:00 menno Exp $
 **/
 
 #ifndef __DRC_H__
@@ -28,7 +39,9 @@ extern "C" {
 
 #define DRC_REF_LEVEL 20*4 /* -20 dB */
 
-void init_drc(drc_info *drc, real_t cut, real_t boost);
+
+drc_info *drc_init(real_t cut, real_t boost);
+void drc_end(drc_info *drc);
 void drc_decode(drc_info *drc, real_t *spec);
 
 
